<?php
namespace Teknisa\EST\Modules\Produto\Controllers;

use Teknisa\EST\Modules\Produto\Services\Produto as ProdutoService;
use Teknisa\EST\Modules\Produto\Services\Fiscal as FiscalService;
use Teknisa\EST\Modules\Produto\Services\ProdutoUtilities;
use Teknisa\Libs\Service\CRUD as CrudService;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Request\Row;
use Zeedhi\Framework\DTO\Request\Filter;
use Zeedhi\Framework\DTO\Response;

class Produto {

    protected $produtoService;
    protected $fiscalService;
    protected $utilities;
    protected $crudService;

    public function __construct(ProdutoService $produtoService, FiscalService $fiscalService,
                                 ProdutoUtilities $utilities, CrudService $crudService) {
        $this->produtoService = $produtoService;
        $this->fiscalService = $fiscalService;
        $this->utilities = $utilities;
        $this->crudService =$crudService;
    }

    public function save(Row $request, Response $response) {
        $row = $request->getRow();
        try {
            $this->utilities->beginTransaction();
            if (isset($row['__is_new']) && $row['__is_new'] === true) {
                $this->produtoService->existArvprod($row['CDARVPROD']);
                $this->rulesToNewProduct($row);
                $row = $this->crudService->getRowsToSave($request, 'produto')[0];
                $this->fiscalService->saveFiscal($row);
                $this->utilities->flush();
            } else {
                $this->rulesToEditProduct($row);
            }
            $this->produtoService->arvprodContainsProdnivante($row['CDARVPROD'], $row['CDPRODNIVANTE']);
            $this->produtoService->correctLevelByArvprod($row['CDARVPROD'], $row['CDNVPRODUTO']);
            $this->utilities->saveInProduto($row);
            $this->utilities->flush();
            $this->utilities->commit();
            $row['__is_new'] = false;
            $dataSet = new DataSet('produto', array($row));
            $response->setDataSets(array($dataSet));
        } catch (\Exception $e) {
            $this->utilities->rollBack();
			$response->setError(new Error($e->getMessage(), $e->getCode()));
		}
    }
    
    private function rulesToEditProduct(&$row) {
        if ($this->produtoService->productType($row) === 'C') {
            $this->produtoService->validateType($row['CDPRODUTO'], 'C');
            $row = $this->produtoService->applyRulesToCompraType($row);
        } else if ($this->produtoService->productType($row) === 'E') {
            $this->produtoService->validateType($row['CDPRODUTO'], 'E');
            $row = $this->produtoService->applyRulesToEstoqueType($row);
        }
    }

    private function rulesToNewProduct(&$row) {
        $row['CDPRODUTO'] = $this->produtoService->getNewCdproduto($row['CDARVPROD']);
        $row['CDNVPRODUTO'] = $this->produtoService->getCdnvprodutoByCdarvprod($row['CDARVPROD']);
        if ($this->produtoService->newProductType($row) === 'C') {
            $row = $this->produtoService->applyRulesToCompraType($row);
        } else if ($this->produtoService->newProductType($row) === 'E') {
            $row = $this->produtoService->applyRulesToEstoqueType($row);
        }
    }
    
    public function produtoFilial(Filter $request, Response $response) {
        $filterCriteria = $request->getFilterCriteria();
        $conditions = $filterCriteria->getConditions();
        $cdFilial = '';
        
        foreach($conditions as $condition) {
            if($condition["columnName"] == "CDFILIAL") {
                $cdFilial = $condition["value"];
                break;
            }
        }
        $cdSegMerc = $this->produtoService->getCdSegMercFilial($cdFilial);

        if($cdSegMerc) {
            $filterSegmento = array("columnName" => "CDSEGMER", "operator" => "=", "value" => $cdSegMerc);
            array_push($conditions, $filterSegmento);
            $dataSourceName = 'produto_filial_segmercado';
        }
        else {
            $dataSourceName = 'produto_filial_semseg';
        }
    
        $dataSet = $this->crudService->getFilteredDataSet($dataSourceName, $conditions, $filterCriteria->getPage(), $filterCriteria->getPageSize());

        $dataSet = new DataSet('produto_filial', $dataSet->getRows());
        $response->setDataSets(array($dataSet));
    }
}